#ifdef _WIN32
#include <windows.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#ifndef __APPLE__
#include <GL/gl.h>
#include <GL/glut.h>
#else
#include <OpenGL/gl.h>
#include <GLUT/glut.h>
#endif
#include <AR/gsub.h>
#include <AR/video.h>
#include <AR/param.h>
#include <AR/ar.h>
#include <math.h>

//
// Camera configuration.
//
#ifdef _WIN32
char			*vconf = "Data\\WDM_camera_flipV.xml";
#else
char			*vconf = "";
#endif

int             xsize, ysize;
int             thresh = 100;
int             count = 0;

char           *cparam_name    = "Data/camera_para.dat";
ARParam         cparam;

char           *patt_name1      = "Data/patt.hiro";
int             patt_id1;
int             patt_1Det;
double          patt_width1     = 80.0;
double          patt_center1[2] = {0.0, 0.0};
double          patt_trans1[3][4];

char           *patt_name2      = "Data/patt.C";
int             patt_2Det;
int             patt_id2;
double          patt_width2     = 80.0;
double          patt_center2[2] = {0.0, 0.0};
double          patt_trans2[3][4];


static void   init(void);
static void   cleanup(void);
static void   keyEvent( unsigned char key, int x, int y);
static void   mainLoop(void);
static void   draw( void );

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	init();

    arVideoCapStart();
    argMainLoop( NULL, keyEvent, mainLoop );
	return (0);
}

static void   keyEvent( unsigned char key, int x, int y)
{
    /* quit if the ESC key is pressed */
    if( key == 0x1b ) {
        printf("*** %f (frame/sec)\n", (double)count/arUtilTimer());
        cleanup();
        exit(0);
    }
    if (key == '+')
		thresh = thresh + 5;
	 if (key == '-')
		thresh = thresh - 5;
	printf("thres = %d\n", thresh);
}

/* main loop */
static void mainLoop(void)
{
    ARUint8         *dataPtr;
    ARMarkerInfo    *marker_info;
    int             marker_num;
    int             i, j, k;

    /* grab a vide frame */
    if( (dataPtr = (ARUint8 *)arVideoGetImage()) == NULL ) {
        arUtilSleep(2);
        return;
    }
    if( count == 0 ) arUtilTimerReset();
    count++;

    argDrawMode2D();
    argDispImage( dataPtr, 0,0 );

    /* detect the markers in the video frame */
    if( arDetectMarker(dataPtr, thresh, &marker_info, &marker_num) < 0 ) {
        cleanup();
        exit(0);
    }

    arVideoCapNext();

	 patt_1Det = 0;
    patt_2Det = 0;
    
	 for (i = 0; i < 2; i++)
	 { 
		 int patt_id = patt_id1;
		 if (i == 1)
			patt_id = patt_id2;
		 /* check for object visibility */
		 k = -1;
		 for( j = 0; j < marker_num; j++ ) {
			  if( patt_id == marker_info[j].id )
			  {
				   if (i == 0)
						patt_1Det = 1;
				   if (i == 1)
						patt_2Det = 1;
					if( k == -1 ) k = j;
					else if( marker_info[k].cf < marker_info[j].cf ) k = j;
			  }
		 }
		 
		 /* get the transformation between the marker and the real camera */
		 if (i == 0 && k != -1) arGetTransMat(&marker_info[k], patt_center1, patt_width1, patt_trans1);
		 if (i == 1 && k != -1) arGetTransMat(&marker_info[k], patt_center2, patt_width2, patt_trans2);
	}
	
	if( !patt_1Det && !patt_2Det )
	{
		  argSwapBuffers();
		  return;
	}
	
    draw();

    argSwapBuffers();
}

static void init( void )
{
    ARParam  wparam;
	
    /* open the video path */
    if( arVideoOpen( vconf ) < 0 ) exit(0);
    /* find the size of the window */
    if( arVideoInqSize(&xsize, &ysize) < 0 ) exit(0);
    printf("Image size (x,y) = (%d,%d)\n", xsize, ysize);

    /* set the initial camera parameters */
    if( arParamLoad(cparam_name, 1, &wparam) < 0 ) {
        printf("Camera parameter load error !!\n");
        exit(0);
    }
    arParamChangeSize( &wparam, xsize, ysize, &cparam );
    arInitCparam( &cparam );
    printf("*** Camera Parameter ***\n");
    arParamDisp( &cparam );

	 patt_1Det = 0;
    if( (patt_id1=arLoadPatt(patt_name1)) < 0 ) {
        printf("pattern 1 load error !!\n");
        exit(0);
    }
   
    patt_2Det = 0; 
    if( (patt_id2=arLoadPatt(patt_name2)) < 0 ) {
        printf("pattern 2 load error !!\n");
        exit(0);
    }

    /* open the graphics window */
    argInit( &cparam, 1.0, 0, 0, 0, 0 );
}

/* cleanup function called when program exits */
static void cleanup(void)
{
    arVideoCapStop();
    arVideoClose();
    argCleanup();
}

static void draw( void )
{
    double    gl_para[16];
    GLfloat   mat_ambient1[]     = {0.0, 0.0, 1.0, 1.0};
    GLfloat   mat_ambient2[]     = {1.0, 1.0, 0.0, 1.0};
    GLfloat   mat_ambient3[]     = {0.0, 1.0, 0.0, 1.0};
    GLfloat   mat_ambient4[]     = {1.0, 0.0, 0.0, 1.0};
    GLfloat   mat_flash[]       = {0.0, 0.0, 1.0, 1.0};
    GLfloat   mat_flash_shiny[] = {50.0};
    GLfloat   light_position[]  = {100.0,-200.0,200.0,0.0};
    GLfloat   ambi[]            = {0.1, 0.1, 0.1, 0.1};
    GLfloat   lightZeroColor[]  = {0.9, 0.9, 0.9, 0.1};
    int i = 0;
    
    argDrawMode3D();
    argDraw3dCamera( 0, 0 );
    glClearDepth( 1.0 );
    glClear(GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    
    int markersClose = 0;
	 if (patt_2Det && patt_1Det && ( fabs(patt_trans2[0][3] - patt_trans1[0][3]) < 10.0f ||  fabs(patt_trans2[1][3] - patt_trans1[1][3]) < 10.0f) )
		  markersClose = 1;
		  
    for (i = 0; i < 2; i++)
    {
		 /* load the camera transformation matrix */
		 if (patt_1Det && i == 0)
		 {
			argConvGlpara(patt_trans1, gl_para);
			printf(" M1 x: %3.1f y: %3.1f z: %3.1f\n", patt_trans1[0][3], patt_trans1[1][3], patt_trans1[2][3]);
		 }
		 if (patt_2Det && i == 1)
		 {
			argConvGlpara(patt_trans2, gl_para);
			printf(" M2 x: %3.1f y: %3.1f z: %3.1f\n", patt_trans2[0][3], patt_trans2[1][3], patt_trans2[2][3]);
		 }
		 
		 glMatrixMode(GL_MODELVIEW);
		 glLoadMatrixd( gl_para );

		 glEnable(GL_LIGHTING);
		 glEnable(GL_LIGHT0);
		 glLightfv(GL_LIGHT0, GL_POSITION, light_position);
		 glLightfv(GL_LIGHT0, GL_AMBIENT, ambi);
		 glLightfv(GL_LIGHT0, GL_DIFFUSE, lightZeroColor);
		 glMaterialfv(GL_FRONT, GL_SPECULAR, mat_flash);
		 glMaterialfv(GL_FRONT, GL_SHININESS, mat_flash_shiny);	
		 if (!markersClose)
		 {
			 if (patt_1Det && i == 0)
				glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient1);	// Color del primer cubo
			 if (patt_2Det && i == 1)
				glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient2);	// Color del segundo cubo
		 }
		 else
			glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient3);	// Color del segundo cubo
		 glMatrixMode(GL_MODELVIEW);
		 glTranslatef( 0.0, 0.0, 25.0 );	// traslación respecto al origen de la marca
		 if ( (patt_1Det && i == 0) || (patt_2Det && i == 1) )
			glutSolidCube(50.0);					// dibuja el cubo cubo
		 
		 glDisable( GL_LIGHTING );
		 glDisable( GL_DEPTH_TEST );
	}
}
